@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -22,22 +22,22 @@ using namespace std;
 using namespace boost;
 using namespace graph_tool;
 
-void generate_maxent_sbm(GraphInterface& gi, boost::any ab,
+void generate_maxent_sbm(GraphInterface& gi, std::any ab,
                          boost::python::object ors, boost::python::object oss,
-                         boost::python::object omrs, boost::any ain_theta,
-                         boost::any aout_theta, bool multi, bool self_loops,
+                         boost::python::object omrs, std::any ain_theta,
+                         std::any aout_theta, bool multi, bool self_loops,
                          rng_t& rng)
 {
     auto rs = get_array<int64_t, 1>(ors);
     auto ss = get_array<int64_t, 1>(oss);
     auto mrs = get_array<double, 1>(omrs);
 
-    typedef vprop_map_t<int32_t>::type bmap_t;
-    auto b = any_cast<bmap_t>(ab).get_unchecked();
+    typedef vprop_map_t<int32_t> bmap_t;
+    auto b = std::any_cast<bmap_t>(ab).get_unchecked();
 
-    typedef vprop_map_t<double>::type dmap_t;
-    auto in_theta = any_cast<dmap_t>(ain_theta).get_unchecked();
-    auto out_theta = any_cast<dmap_t>(aout_theta).get_unchecked();
+    typedef vprop_map_t<double> dmap_t;
+    auto in_theta = std::any_cast<dmap_t>(ain_theta).get_unchecked();
+    auto out_theta = std::any_cast<dmap_t>(aout_theta).get_unchecked();
 
     if (multi)
     {
