@@ -3,7 +3,7 @@
 #
 # graph_tool -- a general graph manipulation python module
 #
-# Copyright (C) 2006-2023 Tiago de Paula Peixoto <tiago@skewed.de>
+# Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -50,6 +50,8 @@ Discrete-time dynamics
    AxelrodState
    BooleanState
    KirmanState
+   NormalState
+   LinearNormalState
 
 Continuous-time dynamics
 ========================
@@ -60,8 +62,23 @@ Continuous-time dynamics
    :template: class.rst
 
    ContinuousStateBase
+   LinearState
+   LVState
    KuramotoState
 
+Belief propagation
+==================
+
+.. autosummary::
+   :nosignatures:
+   :toctree: autosummary
+   :template: class.rst
+
+   BPBaseState
+   GenPottsBPState
+   IsingBPState
+   NormalBPState
+
 
 Contents
 ++++++++
@@ -69,7 +86,7 @@ Contents
 """
 
 from .. import _degree, _prop, Graph, GraphView, _get_rng, PropertyMap, \
-    EdgePropertyMap, VertexPropertyMap, _check_prop_scalar
+    EdgePropertyMap, VertexPropertyMap, _check_prop_scalar, _parallel
 from .. generation import label_self_loops
 import numpy
 import numpy.random
@@ -80,12 +97,16 @@ import scipy.integrate
 from .. dl_import import dl_import
 dl_import("from . import libgraph_tool_dynamics as lib_dynamics")
 
+from . bp import *
+
 __all__ = ["DiscreteStateBase", "EpidemicStateBase", "SIState", "SISState",
            "SIRState", "SIRSState", "VoterState", "MajorityVoterState",
            "BinaryThresholdState", "IsingGlauberState", "CIsingGlauberState",
            "IsingMetropolisState", "PottsGlauberState", "PottsMetropolisState",
-           "AxelrodState", "BooleanState", "KirmanState", "ContinuousStateBase",
-           "KuramotoState"]
+           "AxelrodState", "BooleanState", "KirmanState", "NormalState",
+           "LinearNormalState", "LinearState", "LVState",
+           "GeneralizedBinaryState", "ContinuousStateBase", "KuramotoState",
+           "BPBaseState", "GenPottsBPState", "IsingBPState", "NormalBPState"]
 
 class DiscreteStateBase(object):
     def __init__(self, g, make_state, params, s=None, stype="int32_t"):
@@ -105,13 +126,13 @@ class DiscreteStateBase(object):
 
     def copy(self):
         """Return a copy of the state."""
-        return type(self)(self.g, s=self.s.copy(), **self.params)
+        return type(self)(self.__getstate__())
 
-    def __setstate__(self):
+    def __getstate__(self):
         return dict(g=self.g, s=self.s, params=self.params)
 
-    def __getstate__(self, state):
-        return type(self)(state["g"], s=state["s"], **state["params"])
+    def __setstate__(self, state):
+        self.__init__(**state)
 
     def get_state(self):
         """Returns the internal :class:`~graph_tool.VertexPropertyMap` with the current
@@ -122,18 +143,21 @@ class DiscreteStateBase(object):
         """Returns list of "active" nodes, for states where this concept is used."""
         return self._state.get_active()
 
+    def set_active(self, active):
+        """Sets the list of "active" nodes, for states where this concept is used."""
+        self._state.set_active(numpy.asarray(active, dtype="int64"), _get_rng())
+
     def reset_active(self):
         """Resets list of "active" nodes, for states where this concept is used."""
         self._state.reset_active(_get_rng())
 
+    @_parallel
     def iterate_sync(self, niter=1):
         """Updates nodes synchronously (i.e. a full "sweep" of all nodes in parallel),
         `niter` number of times. This function returns the number of nodes that
         changed state.
 
-        If enabled during compilation, this algorithm runs in parallel
-        (i.e. using more than one thread.)
-
+        @parallel@
         """
         return self._state.iterate_sync(niter, _get_rng())
 
@@ -226,9 +250,10 @@ class SIState(EpidemicStateBase):
         given as follows:
 
         1. If :math:`s_i(t) = 0`, we have :math:`s_i(t+1) = 1` with probability
-            .. math::
 
-               (1-r_i)\left[1-\prod_j(1-\beta_{ij})^{A_{ij}\delta_{s_j(t),1}}\right] + r_i,
+           .. math::
+
+              (1-r_i)\left[1-\prod_j(1-\beta_{ij})^{A_{ij}\delta_{s_j(t),1}}\right] + r_i,
 
            otherwise :math:`s_i(t+1) = 0`.
 
@@ -334,9 +359,10 @@ class SISState(DiscreteStateBase):
         given as follows:
 
         1. If :math:`s_i(t) = 0`, we have :math:`s_i(t+1) = 1` with probability
-            .. math::
 
-               (1-r_i)\left[1-\prod_j(1-\beta_{ij})^{A_{ij}\delta_{s_j(t),1}}\right] + r_i,
+           .. math::
+
+              (1-r_i)\left[1-\prod_j(1-\beta_{ij})^{A_{ij}\delta_{s_j(t),1}}\right] + r_i,
 
            otherwise :math:`s_i(t+1) = 0`.
 
@@ -566,9 +592,10 @@ class SIRSState(DiscreteStateBase):
         given as follows:
 
         1. If :math:`s_i(t) = 0`, we have :math:`s_i(t+1) = 1` with probability
-            .. math::
 
-               (1-r_i)\left[1-\prod_j(1-\beta_{ij})^{A_{ij}\delta_{s_j(t),1}}\right] + r_i,
+           .. math::
+
+              (1-r_i)\left[1-\prod_j(1-\beta_{ij})^{A_{ij}\delta_{s_j(t),1}}\right] + r_i,
 
            otherwise :math:`s_i(t+1) = 0`.
 
@@ -872,18 +899,14 @@ class BinaryThresholdState(DiscreteStateBase):
            >>> state = gt.BinaryThresholdState(g, r=0.25)
            >>> ret = state.iterate_sync(niter=1000)
            >>> gt.graph_draw(g, g.vp.pos, vertex_fill_color=state.s,
-           ...               output="binary-threshold.pdf")
+           ...               output="binary-threshold.svg")
            <...>
 
-        .. testcleanup:: binary-threshold
-
-           conv_png("binary-threshold.pdf")
-
-        .. figure:: binary-threshold.png
+        .. figure:: binary-threshold.svg
            :align: center
            :width: 80%
 
-           State of a binary threshold dynamics on a political blog network.
+           State of a binary threshold dynamics on a :ns:`political blog <polblogs>` network.
         """
 
         if isinstance(w, PropertyMap):
@@ -953,18 +976,14 @@ class IsingGlauberState(DiscreteStateBase):
            >>> state = gt.IsingGlauberState(g, beta=.05)
            >>> ret = state.iterate_async(niter=1000 * g.num_vertices())
            >>> gt.graph_draw(g, g.vp.pos, vertex_fill_color=state.s,
-           ...               output="glauber-ising.pdf")
+           ...               output="glauber-ising.svg")
            <...>
 
-        .. testcleanup:: glauber-ising
-
-           conv_png("glauber-ising.pdf")
-
-        .. figure:: glauber-ising.png
+        .. figure:: glauber-ising.svg
            :align: center
            :width: 80%
 
-           State of a Glauber Ising dynamics on a political blog network.
+           State of a Glauber Ising dynamics on a :ns:`political blog <polblogs>` network.
 
         References
         ----------
@@ -1039,18 +1058,14 @@ class CIsingGlauberState(DiscreteStateBase):
            >>> state = gt.CIsingGlauberState(g, beta=.2)
            >>> ret = state.iterate_async(niter=1000 * g.num_vertices())
            >>> gt.graph_draw(g, g.vp.pos, vertex_fill_color=state.s, vcmap=cm.magma,
-           ...               output="glauber-cising.pdf")
+           ...               output="glauber-cising.svg")
            <...>
 
-        .. testcleanup:: glauber-cising
-
-           conv_png("glauber-cising.pdf")
-
-        .. figure:: glauber-cising.png
+        .. figure:: glauber-cising.svg
            :align: center
            :width: 80%
 
-           State of a continuous Glauber Ising dynamics on a political blog network.
+           State of a continuous Glauber Ising dynamics on a :ns:`political blog <polblogs>` network.
 
         References
         ----------
@@ -1123,18 +1138,14 @@ class IsingMetropolisState(DiscreteStateBase):
            >>> state = gt.IsingMetropolisState(g, beta=.1)
            >>> ret = state.iterate_async(niter=1000 * g.num_vertices())
            >>> gt.graph_draw(g, g.vp.pos, vertex_fill_color=state.s,
-           ...               output="metropolis-ising.pdf")
+           ...               output="metropolis-ising.svg")
            <...>
 
-        .. testcleanup:: metropolis-ising
-
-           conv_png("metropolis-ising.pdf")
-
-        .. figure:: metropolis-ising.png
+        .. figure:: metropolis-ising.svg
            :align: center
            :width: 80%
 
-           State of a Metropolis-Hastings Ising dynamics on a political blog network.
+           State of a Metropolis-Hastings Ising dynamics on a :ns:`political blog <polblogs>` network.
 
         References
         ----------
@@ -1166,8 +1177,8 @@ class IsingMetropolisState(DiscreteStateBase):
                                    dict(w=w, h=h, beta=beta), s)
 
 class PottsGlauberState(DiscreteStateBase):
-    def __init__(self, g, f, w=1, h=0, s=None):
-        r"""Glauber dynamics of the Potts model.
+    def __init__(self, g, f, w=1, h=0, s=None, shift=0):
+        r"""Glauber dynamics of the generalized Potts model.
 
         Parameters
         ----------
@@ -1179,17 +1190,22 @@ class PottsGlauberState(DiscreteStateBase):
         w : :class:`~graph_tool.EdgePropertyMap` or ``float`` (optional, default: ``1.``)
            Edge interaction strength. If a scalar is provided, it's used for all edges.
         h : :class:`~graph_tool.VertexPropertyMap` or iterable or ``float`` (optional, default: ``0.``)
-           Vertex local field. If an iterable is provided, it will be used as
+           Vertex local field. It should be property map of type
+           ``vector<double>``. If an iterable is provided, it will be used as
            the field for all vertices. If a scalar is provided, it will be used
            for all spins values and vertices.
         s : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
            Initial global state. If not provided, a random state will be chosen.
+        shift : ``int`` (optional, default: ``0``)
+           The node states will be shifted be assumed to lie in the range
+           ``[-shift, q-1-shift]``, which will then be translated to the range
+           ``[0, q-1]`` before the transition probabilities are computed.
 
         Notes
         -----
 
-        This implements the Glauber dynamics of the Potts model [potts-model]_
-        on a network.
+        This implements the Glauber dynamics of the generalized Potts model
+        [potts-model]_ on a network.
 
         If a node :math:`i` is updated at time :math:`t`, the transition
         to state :math:`s_i(t+1) \in \{0,\dots,q-1\}` is done with probability
@@ -1216,14 +1232,10 @@ class PottsGlauberState(DiscreteStateBase):
            >>> state = gt.PottsGlauberState(g, f)
            >>> ret = state.iterate_async(niter=1000 * g.num_vertices())
            >>> gt.graph_draw(g, g.vp.pos, vertex_fill_color=state.s,
-           ...               output="glauber-potts.pdf")
+           ...               output="glauber-potts.svg")
            <...>
 
-        .. testcleanup:: glauber-potts
-
-           conv_png("glauber-potts.pdf")
-
-        .. figure:: glauber-potts.png
+        .. figure:: glauber-potts.svg
            :align: center
            :width: 80%
 
@@ -1251,13 +1263,14 @@ class PottsGlauberState(DiscreteStateBase):
                 h = [h] * q
             h = g.new_vp("vector<double>", val=h)
         if s is None:
-            s = g.new_vp("int32_t", vals=numpy.random.randint(0, q, g.num_vertices()))
+            s = g.new_vp("int32_t", vals=numpy.random.randint(shift, shift + q,
+                                                              g.num_vertices()))
         DiscreteStateBase.__init__(self, g,
                                    lib_dynamics.make_potts_glauber_state,
-                                   dict(f=f, w=w, h=h), s)
+                                   dict(f=f, w=w, h=h, shift=shift), s)
 
 class PottsMetropolisState(DiscreteStateBase):
-    def __init__(self, g, f, w=1, h=0, s=None):
+    def __init__(self, g, f, w=1, h=0, s=None, shift=0):
         r"""Metropolis-Hastings dynamics of the Potts model.
 
         Parameters
@@ -1275,6 +1288,10 @@ class PottsMetropolisState(DiscreteStateBase):
            for all spins values and vertices.
         s : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
            Initial global state. If not provided, a random state will be chosen.
+        shift : ``int`` (optional, default: ``0``)
+           The node states will be shifted be assumed to lie in the range
+           ``[-shift, q-1-shift]``, which will then be translated to the range
+           ``[0, q-1]`` before the transition probabilities are computed.
 
         Notes
         -----
@@ -1314,19 +1331,15 @@ class PottsMetropolisState(DiscreteStateBase):
            >>> state = gt.PottsGlauberState(g, f)
            >>> ret = state.iterate_async(niter=1000 * g.num_vertices())
            >>> gt.graph_draw(g, g.vp.pos, vertex_fill_color=state.s,
-           ...               output="metropolis-potts.pdf")
+           ...               output="metropolis-potts.svg")
            <...>
 
-        .. testcleanup:: metropolis-potts
-
-           conv_png("metropolis-potts.pdf")
-
-        .. figure:: metropolis-potts.png
+        .. figure:: metropolis-potts.svg
            :align: center
            :width: 80%
 
            State of a Metropolis-Hastings Potts dynamics with :math:`q=4` on a
-           political blog network.
+           :ns:`political blog <polblogs>` network.
 
         References
         ----------
@@ -1356,10 +1369,11 @@ class PottsMetropolisState(DiscreteStateBase):
                 h = [h] * q
             h = g.new_vp("vector<double>", val=h)
         if s is None:
-            s = g.new_vp("int32_t", vals=numpy.random.randint(0, q, g.num_vertices()))
+            s = g.new_vp("int32_t", vals=numpy.random.randint(shift, shift + q,
+                                                              g.num_vertices()))
         DiscreteStateBase.__init__(self, g,
                                    lib_dynamics.make_potts_metropolis_state,
-                                   dict(f=f, w=w, h=h), s)
+                                   dict(f=f, w=w, h=h, shift=shift), s)
 
 class KirmanState(DiscreteStateBase):
     def __init__(self, g, d=.1, c1=.001, c2=.001, s=None):
@@ -1416,18 +1430,14 @@ class KirmanState(DiscreteStateBase):
            >>> state = gt.KirmanState(g)
            >>> ret = state.iterate_sync(niter=1000)
            >>> gt.graph_draw(g, g.vp.pos, vertex_fill_color=state.s,
-           ...               output="kirman.pdf")
+           ...               output="kirman.svg")
            <...>
 
-        .. testcleanup:: kirman
-
-           conv_png("kirman.pdf")
-
-        .. figure:: kirman.png
+        .. figure:: kirman.svg
            :align: center
            :width: 80%
 
-           State of Kirman's model on a political blog network.
+           State of Kirman's model on a :ns:`political blog <polblogs>` network.
 
         References
         ----------
@@ -1442,6 +1452,19 @@ class KirmanState(DiscreteStateBase):
                                    lib_dynamics.make_kirman_state,
                                    dict(d=d, c1=c1, c2=c2), s)
 
+class GeneralizedBinaryState(DiscreteStateBase):
+    def __init__(self, g, f, r, s=None):
+
+        self.f = numpy.asarray(f, dtype="float")
+        self.r = numpy.asarray(r, dtype="float")
+
+        if s is None:
+            s = g.new_vp("int32_t", vals=numpy.random.randint(0, 2, g.num_vertices()))
+
+        DiscreteStateBase.__init__(self, g,
+                                   lib_dynamics.make_generalized_binary_state,
+                                   dict(f=self.f, r=self.r), s)
+
 class AxelrodState(DiscreteStateBase):
     def __init__(self, g, f=10, q=2, r=0, s=None):
         r"""Axelrod's culture dissemination model.
@@ -1509,18 +1532,14 @@ class AxelrodState(DiscreteStateBase):
            >>> ret = state.iterate_async(niter=10000000)
            >>> gt.graph_draw(g, g.vp.pos,
            ...               vertex_fill_color=gt.perfect_prop_hash([state.s])[0],
-           ...               vcmap=cm.magma, output="axelrod.pdf")
+           ...               vcmap=cm.magma, output="axelrod.svg")
            <...>
 
-        .. testcleanup:: axelrod
-
-           conv_png("axelrod.pdf")
-
-        .. figure:: axelrod.png
+        .. figure:: axelrod.svg
            :align: center
            :width: 80%
 
-           State of Axelrod's model on a political blog network.
+           State of Axelrod's model on a :ns:`political blog <polblogs>` network.
 
         References
         ----------
@@ -1594,7 +1613,7 @@ class BooleanState(DiscreteStateBase):
            ...     ret = state.iterate_sync(niter=1)
            ...     l += 1
            >>> print("Period length:", l)
-           Period length: 4
+           Period length: 9
 
         """
 
@@ -1607,6 +1626,179 @@ class BooleanState(DiscreteStateBase):
                                    dict(f=f, p=p, r=r), s,
                                    stype="bool")
 
+class NormalState(DiscreteStateBase):
+    def __init__(self, g, w=0, sigma=1, s=None):
+        r"""Multivariate Normal distribution.
+
+        Parameters
+        ----------
+        g : :class:`~graph_tool.Graph`
+           Graph represening the conditional dependencies.
+        w : :class:`~graph_tool.EdgePropertyMap` or ``float`` (optional, default: ``0``)
+           Inverse covariance (i.e. coupling strength) between nodes.
+        sigma : :class:`~graph_tool.VertexPropertyMap` or ``float`` (optional, default: ``1``)
+           Node standard deviation.
+        s : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
+           Initial global state. If not provided, a random state will be chosen.
+
+        Notes
+        -----
+
+        This implements a zero-mean multivariate Normal distribution.
+
+        If a node :math:`i` is updated at time :math:`t`, the transition
+        to state :math:`s_i(t+1)` is given by
+
+        .. math::
+
+           P(s_i(t+1)|\boldsymbol s(t), \boldsymbol A, \boldsymbol w, \boldsymbol \sigma)
+           = \frac{\exp\left[-\frac{\left(s_i(t+1)+\sigma_i^2\sum_jA_{ij}w_{ij}s_j(t)\right)^2}
+                                   {2\sigma_i^2}\right]}
+                  {\sqrt{2\pi}\sigma_i}
+
+        which will lead, asymptotically with :math:`t\to\infty`, to a zero-mean
+        multivariate Normal distribution:
+
+        .. math::
+
+           P(\boldsymbol s | \boldsymbol W) =
+           \frac{\mathrm{e}^{-\frac{1}{2} {\boldsymbol x}^{\top}\boldsymbol W \boldsymbol x}}
+                {\sqrt{(2\pi)^N |\boldsymbol W^{-1}|}},
+
+        where :math:`W_{ij}=w_{ij}` for :math:`i\neq j` and :math:`W_{ii}=1/\sigma_i^2`.
+
+        Examples
+        --------
+
+        .. testsetup:: polblogs-normal
+
+           gt.seed_rng(42)
+           np.random.seed(42)
+
+        .. doctest:: polblogs-normal
+
+           >>> g = gt.GraphView(gt.collection.data["polblogs"].copy(), directed=False)
+           >>> gt.remove_parallel_edges(g)
+           >>> g = gt.extract_largest_component(g, prune=True)
+           >>> state = gt.NormalState(g, sigma=0.001, w=-100)
+           >>> ret = state.iterate_sync(niter=1000)
+           >>> gt.graph_draw(g, g.vp.pos, vertex_fill_color=state.s,
+           ...               output="polblogs-normal.svg")
+           <...>
+
+        .. figure:: polblogs-normal.svg
+           :align: center
+           :width: 80%
+
+           Sample of a multivariate Normal on a :ns:`political blog <polblogs>` network.
+
+        References
+        ----------
+        .. [normal] https://en.wikipedia.org/wiki/Multivariate_normal_distribution
+        """
+
+        if isinstance(w, PropertyMap):
+            if w.value_type() != "double":
+                w = w.copy("double")
+        else:
+            w = g.new_ep("double", val=w)
+        if isinstance(sigma, PropertyMap):
+            if sigma.value_type() != "double":
+                sigma = sigma.copy("double")
+        else:
+            sigma = g.new_vp("double", val=sigma)
+        if s is None:
+            s = g.new_vp("double")
+        DiscreteStateBase.__init__(self, g,
+                                   lib_dynamics.make_normal_state,
+                                   dict(w=w, sigma=sigma), s, stype="double")
+
+class LinearNormalState(DiscreteStateBase):
+    def __init__(self, g, w=0, sigma=1, s=None):
+        r"""Linear distrecte-time dynamical system with noise.
+
+        Parameters
+        ----------
+        g : :class:`~graph_tool.Graph`
+           Graph to be used for the dynamics
+        w : :class:`~graph_tool.EdgePropertyMap` or ``float`` (optional, default: ``1``)
+           Coupling strength of each edge. If a scalar is given, it will be
+           used for all edges.
+        sigma : ``float`` (optional, default: ``.0``)
+           Stochastic noise magnitude.
+        s : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
+           Initial global state. If not provided, a random state will be chosen.
+
+        Notes
+        -----
+
+        This implements a distrecte time linear dynamical system with noise,
+        i.e. each node has an variable :math:`s_i`, which evolves in time
+        according to a Markov chain with transitio probabilities:
+
+        .. math::
+
+           P(s_i(t+1)|\boldsymbol s, \boldsymbol A, \boldsymbol w, \sigma)
+           = \frac{\exp\left[-\frac{\left(s_i(t+1)-\sum_{j}A_{ij}w_{ij}s_j\right)^2}{2\sigma^2}\right]}{\sqrt{2\pi}\sigma}
+
+        Examples
+        --------
+
+        .. testsetup:: linear-discrete
+
+           gt.seed_rng(49)
+           np.random.seed(49)
+
+        .. doctest:: linear-discrete
+
+
+           >>> g = gt.collection.data["karate"].copy()
+           >>> s = g.new_vp("double", np.random.normal(0, 1, g.num_vertices()))
+           >>> w = g.new_ep("double", vals=np.random.normal(0, .1, g.num_edges()))
+           >>> state = gt.LinearNormalState(g, s=s, w=w)
+           >>> ss = []
+           >>> for t in range(10):
+           ...     ret = state.iterate_sync()
+           ...     ss.append(state.get_state().fa.copy())
+
+           >>> figure(figsize=(6, 4))
+           <...>
+           >>> for v in g.vertices():
+           ...    plot(np.arange(len(ss)), [s[int(v)] for s in ss], "-o")
+           [...]
+           >>> xlabel(r"Time")
+           Text(...)
+           >>> ylabel(r"$s_i$")
+           Text(...)
+           >>> tight_layout()
+           >>> savefig("karate-linear-discrete.svg")
+
+        .. figure:: karate-linear-discrete.svg
+           :align: center
+
+           Linear dynamics on the :ns:`Karate Club <karate>` network.
+
+        References
+        ----------
+        .. [linear] https://en.wikipedia.org/wiki/Linear_dynamical_system
+
+        """
+
+        if isinstance(w, PropertyMap):
+            if w.value_type() != "double":
+                w = w.copy("double")
+        else:
+            w = g.new_ep("double", val=w)
+        if isinstance(sigma, PropertyMap):
+            if sigma.value_type() != "double":
+                sigma = sigma.copy("double")
+        else:
+            sigma = g.new_vp("double", val=sigma)
+        if s is None:
+            s = g.new_vp("double")
+        DiscreteStateBase.__init__(self, g,
+                                   lib_dynamics.make_linear_normal_state,
+                                   dict(w=w, sigma=sigma), s, stype="double")
 
 class ContinuousStateBase(object):
     def __init__(self, g, make_state, params, t0=0, s=None, stype="double"):
@@ -1625,26 +1817,27 @@ class ContinuousStateBase(object):
                                  _prop("v", g, self.s_diff), params, _get_rng())
 
     def copy(self):
-        r"""Copy state."""
-        return type(self)(g, s=self.s.copy(), **self.params)
+        """Return a copy of the state."""
+        return type(self)(**self.__getstate__())
 
-    def __setstate__(self):
+    def __getstate__(self):
         return dict(g=self.g, s=self.s, params=self.params)
 
-    def __getstate__(self, state):
-        return type(self)(state["g"], s=state["s"], **state["params"])
+    def __setstate__(self, state):
+        self.__init__(**state)
 
     def get_state(self):
         r"""Returns the internal :class:`~graph_tool.VertexPropertyMap` with the current
         state."""
         return self.s
 
+    @_parallel
     def get_diff(self, dt):
         r"""Returns the current time derivative for all the nodes. The parameter ``dt``
         is the time interval in consideration, which is used only if the ODE has
         a stochastic component.
 
-        If enabled during compilation, this algorithm runs in parallel.
+        @parallel@
         """
         self._state.get_diff_sync(self.t, dt, _get_rng())
         return self.s_diff.fa
@@ -1678,6 +1871,191 @@ class ContinuousStateBase(object):
             self.t = t
             self.s.fa += self.get_diff(dt) * dt
 
+class LinearState(ContinuousStateBase):
+    def __init__(self, g, w=1, sigma=0, t0=0, s=None):
+        r"""Linear dynamical system with noise.
+
+        Parameters
+        ----------
+        g : :class:`~graph_tool.Graph`
+           Graph to be used for the dynamics
+        w : :class:`~graph_tool.EdgePropertyMap` or ``float`` (optional, default: ``1``)
+           Coupling strength of each edge. If a scalar is given, it will be
+           used for all edges.
+        sigma : ``float`` (optional, default: ``.0``)
+           Stochastic noise magnitude.
+        s : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
+           Initial global state. If not provided, a random state will be chosen.
+
+        Notes
+        -----
+
+        This implements a linear dynamical system with noise, i.e. each node has
+        an variable :math:`s_i`, which evolves in time obeying the
+        differential equation:
+
+        .. math::
+
+           \frac{\mathrm{d}s_i}{\mathrm{d}t} = \sum_{j}A_{ij}w_{ij}s_j + \sigma\xi_i(t),
+
+        where :math:`\xi_i(t)` is a Gaussian noise with zero mean and unit
+        variance (implemented according to the Itô definition).
+
+        Examples
+        --------
+
+        .. testsetup:: linear
+
+           gt.seed_rng(49)
+           np.random.seed(49)
+
+        .. doctest:: linear
+
+
+           >>> g = gt.collection.data["karate"].copy()
+           >>> s = g.new_vp("double", np.random.normal(0, 1, g.num_vertices()))
+           >>> w = g.new_ep("double", vals=np.random.normal(0, .1, g.num_edges()))
+           >>> state = gt.LinearState(g, s=s, w=w)
+           >>> ss = []
+           >>> ts = linspace(0, 10, 1000)
+           >>> for t in ts:
+           ...     ret = state.solve(t, first_step=0.0001)
+           ...     ss.append(state.get_state().fa.copy())
+
+           >>> figure(figsize=(6, 4))
+           <...>
+           >>> for v in g.vertices():
+           ...    plot(ts, [s[int(v)] for s in ss])
+           [...]
+           >>> xlabel(r"Time")
+           Text(...)
+           >>> ylabel(r"$s_i$")
+           Text(...)
+           >>> tight_layout()
+           >>> savefig("karate-linear.svg")
+
+        .. figure:: karate-linear.svg
+           :align: center
+
+           Linear dynamics on the :ns:`Karate Club <karate>` network.
+
+        References
+        ----------
+        .. [linear] https://en.wikipedia.org/wiki/Linear_dynamical_system
+        """
+        if not isinstance(sigma, PropertyMap):
+            sigma = g.new_vp("double", val=sigma)
+        elif sigma.value_type() != "double":
+            sigma = sigma.copy("double")
+        if not isinstance(w, PropertyMap):
+            w = g.new_ep("double", val=w)
+        elif w.value_type() != "double":
+            w = w.copy("double")
+        if s is None:
+            s = g.new_vp("double", vals=numpy.random.random(g.num_vertices()))
+
+        ContinuousStateBase.__init__(self, g, lib_dynamics.make_linear_state,
+                                     dict(w=w, sigma=sigma), t0, s)
+
+
+class LVState(ContinuousStateBase):
+    def __init__(self, g, w=1, r=1, sigma=0, mig=0, t0=0, s=None):
+        r"""Generalized Lotka-Volterra model.
+
+        Parameters
+        ----------
+        g : :class:`~graph_tool.Graph`
+           Graph to be used for the dynamics
+        w : :class:`~graph_tool.EdgePropertyMap` or ``float`` (optional, default: ``1``)
+           Coupling strength of each edge. If a scalar is given, it will be
+           used for all edges.
+        r : :class:`~graph_tool.VertexPropertyMap` or ``float`` (optional, default: ``1``)
+           Intrinsic birth or death rates. If a scalar is given, it will be
+           used for all nodes.
+        sigma : ``float`` (optional, default: ``.0``)
+           Stochastic noise magnitude.
+        s : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
+           Initial global state. If not provided, a random state will be chosen.
+
+        Notes
+        -----
+        This implements a generalized Lotka-Volterra (gLV) dynamical system with
+        demographic noise, i.e. each node has an variable :math:`s_i`, which
+        evolves in time obeying the differential equation:
+
+        .. math::
+
+           \frac{\mathrm{d}s_i}{\mathrm{d}t} = s_i\left(r_i + \sum_{j}A_{ij}w_{ij}s_j\right) + \sigma\sqrt{s_i}\xi_i(t),
+
+        where :math:`\xi_i(t)` is a Gaussian noise with zero mean and unit
+        variance (implemented according to the Itô definition).
+
+        Examples
+        --------
+
+        .. testsetup:: glv
+
+           gt.seed_rng(42)
+           np.random.seed(42)
+
+        .. doctest:: glv
+
+           >>> g = gt.collection.data["karate"].copy()
+           >>> s = g.new_vp("double", vals=1 + 10 * np.random.random(g.num_vertices()))
+           >>> r = g.new_vp("double", val=-1)
+           >>> w = g.new_ep("double", vals=np.random.normal(0, .5, g.num_edges()))
+           >>> for v in g.vertices():
+           ...     e = g.add_edge(v,v)
+           ...     w[e] = -.7
+           >>> state = gt.LVState(g, s=s, r=r, w=w)
+           >>> ss = []
+           >>> ts = linspace(0, .4, 1000)
+           >>> for t in ts:
+           ...     ret = state.solve(t, first_step=1e-6)
+           ...     ss.append(state.get_state().fa.copy())
+           >>> figure(figsize=(6, 4))
+           <...>
+           >>> for v in g.vertices():
+           ...    plot(ts, [s[int(v)] for s in ss])
+           [...]
+           >>> xlabel(r"Time")
+           Text(...)
+           >>> ylabel(r"$s_i$")
+           Text(...)
+           >>> tight_layout()
+           >>> savefig("karate-glv.svg")
+
+        .. figure:: karate-glv.svg
+           :align: center
+
+           gLV dynamics on the :ns:`Karate Club <karate>` network.
+
+        References
+        ----------
+        .. [glv] https://en.wikipedia.org/wiki/Generalized_Lotka%E2%80%93Volterra_equation
+
+        """
+        if not isinstance(sigma, PropertyMap):
+            sigma = g.new_vp("double", val=sigma)
+        elif sigma.value_type() != "double":
+            sigma = sigma.copy("double")
+        if not isinstance(r, PropertyMap):
+            r = g.new_vp("double", val=r)
+        elif r.value_type() != "double":
+            r = r.copy("double")
+        if not isinstance(mig, PropertyMap):
+            mig = g.new_vp("double", val=mig)
+        elif mig.value_type() != "double":
+            mig = mig.copy("double")
+        if not isinstance(w, PropertyMap):
+            w = g.new_ep("double", val=w)
+        elif w.value_type() != "double":
+            w = w.copy("double")
+        if s is None:
+            s = g.new_vp("double", vals=numpy.random.random(g.num_vertices()))
+
+        ContinuousStateBase.__init__(self, g, lib_dynamics.make_LV_state,
+                                     dict(w=w, r=r, sigma=sigma, mig=mig), t0, s)
 
 class KuramotoState(ContinuousStateBase):
     def __init__(self, g, omega=1, w=1, sigma=0, t0=0, s=None):
@@ -1693,8 +2071,9 @@ class KuramotoState(ContinuousStateBase):
         w : :class:`~graph_tool.EdgePropertyMap` or ``float`` (optional, default: ``1``)
            Coupling strength of each edge. If a scalar is given, it will be
            used for all edges.
-        sigma : ``float`` (optional, default: ``.0``)
-           Stochastic noise magnitude.
+        sigma : :class:`~graph_tool.VertexPropertyMap` or ``float`` (optional, default: ``0``)
+           Stochastic noise magnitude for each node. If a scalar is given, it will
+           be used for all nodes.
         s : :class:`~graph_tool.VertexPropertyMap` (optional, default: ``None``)
            Initial global state. If not provided, a random state will be chosen.
 
@@ -1724,7 +2103,6 @@ class KuramotoState(ContinuousStateBase):
 
         .. doctest:: kuramoto
 
-
            >>> g = gt.collection.data["karate"]
            >>> omega = g.new_vp("double", np.random.normal(0, 1, g.num_vertices())) 
            >>> state = gt.KuramotoState(g, omega=omega, w=1.5)
@@ -1749,7 +2127,7 @@ class KuramotoState(ContinuousStateBase):
         .. figure:: karate-kuramoto.svg
            :align: center
 
-           Kuramoto oscillator dynamics on the Karate Club network.
+           Kuramoto oscillator dynamics on the :ns:`Karate Club <karate>` network.
 
         References
         ----------
@@ -1769,6 +2147,10 @@ class KuramotoState(ContinuousStateBase):
             omega = g.new_vp("double", val=omega)
         elif omega.value_type() != "double":
             omega = omega.copy("double")
+        if not isinstance(sigma, PropertyMap):
+            sigma = g.new_vp("double", val=sigma)
+        elif sigma.value_type() != "double":
+            sigma = sigma.copy("double")
         if not isinstance(w, PropertyMap):
             w = g.new_ep("double", val=w)
         elif w.value_type() != "double":
